from PySZG import *
import random
import time

# This class implements simple navigation. It is based on polling (though you could also come up
# with a navigator class based on input events, like the manipulator class below).
# The user travels through the scene following the wand direction by pushing the joystick forward
# and orthogonally to the wand direction by pushing the joystick from side to side.
# The class makes assumptions about the input device. Comments in the code explain what they are.
class navigator:
	# Constructor.
	def __init__(self):
		# The scene graph "navigation node". All geometry that will be effected by navigation
		# must be attached below this node. Note that it is possible to have some geometry
		# that does not move during navigation (think of a head's-up display or a tool the
		# user has).
		self.node = None
		# Must keep track of the time between updates so we can move at a fixed speed.
		self.lastTime = -1
		# The speed in units per second.
		self.speed = 1.0
		# The code assumes joystick axis values range between -1 and 1. Do not navigate
		# unless the absolute value of an axis is greater than threshold.
		self.threshold = 0.4

	# This function is called once per event loop. Thus, the navigator class works by "polling".
	# Must pass in the framework object (which gives us access to current input values) and the
	# current time.
	def update(self, framework, time):
		# If this is the first time calling the method, just set the time called and return.
		if self.lastTime < 0:
			self.lastTime = time
			return
		# Assumes that the "neutral" position of the wand (which is matrix 1) is pointed straight at 
		# the front CAVE wall (and that we are using the CAVE coordinate system, whereby a front pointing vector would
		# be (0,0,-1) ).
		# ar_ERM is shorthand for ar_extractRotationMatrix. This is related to some other calls ar_ETM (ar_extractTranslationMatrix)
		# and ar_ESM (ar_extractScaleMatrix). If a matrix is assumed to be the product of T*R*S (translation, rotation, and scale 
		# where the scale matrix is assumed to be uniform), then we can easily compute the decomposition, which is what these
		# functions do.
		wandDir = ar_ERM(framework.getMatrix(1))*arVector3(0,0,-1)
		# In the CAVE coordinate system, (1,0,0) would be to the right. Again, we "extract" the rotation from the wand's transform.
		wandTransDir = ar_ERM(framework.getMatrix(1))*arVector3(1,0,0)
		# Figure out the navigation deltas, based on the joystick values. This is another assumption 
		# about our input device. It should have two axes, each reporting values between -1 and 1. The
		# first axis should be horizontal and report 1 when pushed to the right. The second axis
		# should be vertical and report 1 when pushed forward.
		# To get around problems with the joystick not reporting exactly (0,0) in its neutral position,
		# there must be an action threshold.
		deltaX = framework.getAxis(0)
		deltaY = framework.getAxis(1)
		# Note how the joystick response above the threshold value gets scaled to give a full range.
		if abs(deltaX) < self.threshold:
			deltaX = 0
		else:
			deltaX = (deltaX - self.threshold) / (1.0 - self.threshold)
		if abs(deltaY) < self.threshold:
			deltaY = 0
		else:
			deltaY = (deltaY - self.threshold) / (1.0 - self.threshold)
		# t is the egocentric translation vector.
		t = (wandDir * deltaY + wandTransDir * deltaX) * (time - self.lastTime) * self.speed 
		# Since we are (in the scene graph context) moving the world instead, we must apply the inverse
		# transformation (egocentric motion to exocentric motion). ar_TM is shorthand for ar_TranslationMatrix and
		# returns an appropriate arMatrix4 from its arVector3 input.
		self.node.set(ar_TM(-t)*self.node.get())
		self.lastTime = time

#Used to read in input
class tool:
    def __init__(self):
        #These are set true when a button is pressed
        #Must be reset outside of this class
	self.pressLeft = False
        self.pressRight = False
        self.pressLeft2 = False
        self.pressRight2 = False

    #Called by eventprocess
    def update(self, framework, event):
        #Only respond to button events when it is a
        #Mousedown event
	if event.getType() == AR_EVENT_BUTTON and event.getButton() == 0:
	    #getIndex = which button
            #In input simulator, different button sets
            #are selected by hitting space bar while in
            #mode 4 or 5.
	    if event.getIndex() == 0:
		self.pressLeft = True
	    elif event.getIndex() == 1:
		self.pressRight = True
            elif event.getIndex() == 2:
		self.pressLeft2 = True
	    elif event.getIndex() == 3:
		self.pressRight2 = True

    #SZG Framework calls this with an eventqueue
    def eventProcessing(self, framework, eventQueue):
        while not eventQueue.empty():
            e = eventQueue.popNextEvent()
            self.update(framework, e)
        return 1

f = arDistSceneGraphFramework();

#Make the tool so we can read input
theTool = tool()

#Set the event queue callback
f.setEventQueueCallback(theTool.eventProcessing)

if f.init(sys.argv) != 1:
    #Oh no....framework failed to initialize
    sys.exit()

#Must be operating a phleet
if f.getStandalone():
    print("This program only works in cluster mode. Please log in.")
    sys.exit()

#This is where the fun stuff begins
peer = arGraphicsPeer()
peer.setName("world1")
peer.queueData(0)
peer.init(f.getSZGClient())
#peer.setTexturePath("c:/szg-1.0/doc/python/texture")
peer.setTexturePath(f.getSZGClient().getAttribute("SZG_RENDER","texture_path"))
peer.loadAlphabet(f.getSZGClient().getAttribute("SZG_RENDER","text_path"))
#Set the peer to use our database
peer.setBridge(f.getDatabase())

worldRoot = f.getDatabase().getRoot()

nav = navigator()
nav.node = worldRoot.new("transform")

#Set the root node from which we share
#peer.setBridgeRootMapNode(nav.node)
peer.setBridgeRootMapNode(nav.node)

if peer.start() != 1:
    sys.exit()
if f.start() != 1:
    sys.exit()

if peer.connectToPeer("world0") < 0:
    print("Failed to connect to specified peer.")
    sys.exit()
else:
    print("Downloading from peer.")
    if peer.pullSerial("world0", 0, 0, AR_TRANSIENT_NODE, AR_TRANSIENT_NODE, AR_TRANSIENT_NODE) == 0:
	print("Error in downloading")

worldRoot = peer.find("world1")

reverseNav = worldRoot.new("transform")
reverseNav.set(nav.node.get().inverse())
balltrans = reverseNav.new("transform")

ballY = 5
ballV = 0

balltrans.set(ar_TM(0, ballY, 0))
ballmat = balltrans.new("material")
mat = arMaterial();
mat.diffuse = arVector3(1,0,0)
ballmat.set(mat)
ballvis = ballmat.new("visibility")
ballvis.set(1)
ballvis.setNodeLevel(AR_TRANSIENT_NODE)
sphere = arSphereMesh(50)
sphere.attachMesh(ballvis)

bounce = False
filtering = False

balltrans.setNodeLevel(AR_TRANSIENT_NODE)
peer.localFilterDataBelow("world0", ballvis.getID(), AR_STABLE_NODE)
ballvis.set(0)

while 1:
    f.setViewer()
    f.setPlayer()
    f.processEventQueue()

    nav.update(f, time.clock())
    reverseNav.set(nav.node.get().inverse())
	
    if theTool.pressLeft:
        theTool.pressLeft = False
        mat = arMaterial();
        mat.diffuse = arVector3(random.random(), random.random(), random.random())
        ballmat.set(mat)

    if theTool.pressRight:
        theTool.pressRight = False
        bounce = not bounce

    if bounce:
        ballY = ballY + ballV*.001
        ballV = ballV - 9.8*.001
        if ballY < 1.05:
            ballV = -ballV
        balltrans.set(ar_TM(3, ballY, -5))

    if theTool.pressLeft2:
        theTool.pressLeft2 = False
        filtering = not filtering
        if filtering:
            peer.remoteFilterDataBelow("world0", 0, AR_STABLE_NODE)
        else:
            peer.remoteFilterDataBelow("world0", 0, AR_TRANSIENT_NODE)
        
    f.loadNavMatrix()
    #time.sleep(0.01)

                                                                                                                                                                                                                                                                                                                                                                                                                                                         
